#include <stdio.h>
#include <curl/curl.h>
#include <string>

#include <utime.h>
#include <sys/stat.h>

#include <iostream>
#include <fstream>
#include "ftpparse.h"

#include "curlget.h"


/* this is how the CURLOPT_XFERINFOFUNCTION callback works */
static int xferinfo(void *p,
                    curl_off_t dltotal, curl_off_t dlnow,
                    curl_off_t ultotal, curl_off_t ulnow)
{
  fprintf(stderr, "UP: %" CURL_FORMAT_CURL_OFF_T " of %" CURL_FORMAT_CURL_OFF_T
          "  DOWN: %" CURL_FORMAT_CURL_OFF_T " of %" CURL_FORMAT_CURL_OFF_T
          "\r",
          ulnow, ultotal, dlnow, dltotal);
  return 0;
}

/* for libcurl older than 7.32.0 (CURLOPT_PROGRESSFUNCTION) */
static int older_progress(void *p,
                          double dltotal, double dlnow,
                          double ultotal, double ulnow)
{
  return xferinfo(p,
                  (curl_off_t)dltotal,
                  (curl_off_t)dlnow,
                  (curl_off_t)ultotal,
                  (curl_off_t)ulnow);
}
 

int downloadFile(const char *url, const char *outfilename,bool keepServerTimestamp)
{
  CURL *curl;
  FILE *fp;
  //FILE *respfile;
  CURLcode res;
  long filetime = -1;
  struct stat attrib;
  curl = curl_easy_init();
  if (curl)
    {
      fp = fopen(outfilename,"wb");
      curl_easy_setopt(curl, CURLOPT_URL, url);
      curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION, fwrite);
      curl_easy_setopt(curl, CURLOPT_WRITEDATA, fp);
      curl_easy_setopt(curl, CURLOPT_VERBOSE, 0L);
      curl_easy_setopt(curl, CURLOPT_FILETIME, 1L);
      curl_easy_setopt(curl, CURLOPT_FAILONERROR, 1L);
      //respfile = fopen("ftp-responses", "wb");
      //curl_easy_setopt(curl, CURLOPT_HEADERFUNCTION, fwrite);
      //curl_easy_setopt(curl, CURLOPT_HEADERDATA, respfile);
      curl_easy_setopt(curl, CURLOPT_NOPROGRESS, 0L);
      curl_easy_setopt(curl, CURLOPT_PROGRESSFUNCTION, older_progress);
      // #if LIBCURL_VERSION_NUM >= 0x072000
      // 	// CURLOPT_XFERINFO* was introduced in 7.32.0, no earlier libcurl versions will compile as they won't have the symbols around.
      // 	curl_easy_setopt(curl, CURLOPT_XFERINFOFUNCTION, xferinfo);
      // #endif
      fprintf(stderr, "Connecting...\r");
      res = curl_easy_perform(curl);
      fprintf(stderr, "\n");
      if(res != CURLE_OK) curl_easy_strerror(res);

      if(CURLE_OK == res) {
        curl_easy_getinfo(curl, CURLINFO_FILETIME, &filetime);
      }
	
      curl_easy_cleanup(curl);
      fclose(fp);
      //fclose(respfile);

      if(keepServerTimestamp && filetime>0){
        if(!stat(outfilename, &attrib)){
          struct utimbuf serverTimestamp;
          serverTimestamp.actime=attrib.st_atime;
          serverTimestamp.modtime=filetime;
          utime(outfilename,&serverTimestamp);
        }
      }
    }
  return res == CURLE_OK;
}

int listFtp(const char *url)
{
  CURL *curl;
  CURLcode res;
  FILE *ftpfile;
  //FILE *respfile;

  curl = curl_easy_init();
  if(curl) {
    ftpfile = fopen("ftp-list", "wb");
    curl_easy_setopt(curl, CURLOPT_URL, url);
    curl_easy_setopt(curl, CURLOPT_WRITEDATA, ftpfile);
    curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION, fwrite);
    curl_easy_setopt(curl, CURLOPT_VERBOSE, 0L);
    curl_easy_setopt(curl, CURLOPT_FAILONERROR, 1L);
    //respfile = fopen("ftp-responses", "wb");
    //curl_easy_setopt(curl, CURLOPT_HEADERFUNCTION, fwrite);
    //curl_easy_setopt(curl, CURLOPT_HEADERDATA, respfile);
    fprintf(stderr, "Connecting...\r");
    res = curl_easy_perform(curl);
    fprintf(stderr, "Connecting... ok.\n");
    if(res != CURLE_OK) curl_easy_strerror(res);

    curl_easy_cleanup(curl);
    fclose(ftpfile);
    //fclose(respfile);
  }

  return res == CURLE_OK;
}

std::string readFtpList(std::string searchnamebeg, std::string searchnameend){
  std::string line;
  std::ifstream myfile("ftp-list");
  struct ftpparse ftpp;
  std::string selectedfilename;
  long long largestsize=0;
  long long newesttime=0;
  if(myfile.is_open()){
    while(getline(myfile,line)){
      ftpparse(&ftpp,line.data(),line.length());
      //ftpp.name ist not necessarily 0-terminated, thus print with given length
      //printf("%*.*s %9li %9Li\n",0,ftpp.namelen,ftpp.name,ftpp.size,(long long)ftpp.mtime);
      if(searchnamebeg.length()>0){
        std::string filename(ftpp.name,ftpp.namelen);
        if(filename.compare(0,searchnamebeg.length(),searchnamebeg)==0 &&
           filename.compare(filename.length()-searchnameend.length(),searchnameend.length(),searchnameend)==0){
          //printf("%*.*s %9li %9Li\n",0,ftpp.namelen,ftpp.name,ftpp.size,(long long)ftpp.mtime);
          if((long long)ftpp.mtime>newesttime){
            newesttime=(long long)ftpp.mtime;
            largestsize=ftpp.size;
            selectedfilename=filename;
            //printf("1: %*.*s %9li %9Li\n",0,ftpp.namelen,ftpp.name,ftpp.size,(long long)ftpp.mtime);
          }else if((long long)ftpp.mtime==newesttime && ftpp.size>largestsize){
            newesttime=(long long)ftpp.mtime;
            largestsize=ftpp.size;
            selectedfilename=filename;	    
            //printf("2: %*.*s %9li %9Li\n",0,ftpp.namelen,ftpp.name,ftpp.size,(long long)ftpp.mtime);
          }
        }
      }else{
        printf("%*.*s %9li %9Li\n",0,ftpp.namelen,ftpp.name,ftpp.size,(long long)ftpp.mtime);
      }
    }
    myfile.close();
  }
  return selectedfilename;
}

