#include <stdio.h>
#include <math.h>
#include <time.h>
#include <stdlib.h>
#include <iostream>
#include <fstream>
#include <string>
#include <sys/stat.h>
#include <unistd.h>
#include <limits>

#include "curlget.h"
#include "sofa.h"
#include "sofam.h"
#include "calceph.h"

void DownloadFiles();
void DownloadEphemerides();
void ShowConstants(t_calcephbin *peph);
double ReadTTcorr(double mjd_utc);
double ReadDAT(double jd_utc, double mjd_utc);double ReadDAT(double jd_utc, double mjd_utc);
double ReadDUT(double mjd, double &xp, double &yp, double &dX, double &dY);
double ReadDT(double year);

// run with current local time:  ./thaec
// run with some given UTC time: ./thaec 21.10.2014 18:23:50
// run with some given UTC time: ./thaec 2015/10/21 18:23:50
int main(int argc, char* argv[])
{
  // point towards the ephemeris file, only the big one (not the headers or such)
  char *ephemerisDataFiles[]={ "data/inpop13c_TDB_m100_p100_tt.dat" };
  // examples:
  //char *ephemerisDataFile="/software/data/ephemeris/DE430/linux_p1550p2650.430";
  //char *ephemerisDataFile="/software/data/ephemeris/inpop13c_TDB_m1000_p1000_tt.dat";

  DownloadFiles();printf("\n");
  DownloadEphemerides();printf("\n");

  t_calcephbin *peph=calceph_open_array(1,ephemerisDataFiles);
  if(!peph){
    printf("cannot open ephemerides files...\n");
    return 1;
  }
  //ShowConstants(peph);

  // your location on earth (east is +), elevation (m), temperature (deg C), pressure (hPa), and humidity (%), wavelength (micrometer, 0.38 to 0.78 for visible light, 0.78-100 for IR and exactly 101 for radio)
  double lat=50.11552, lon=8.68417, elev=113, temp=20, press=1013, relhum=0.5, wavelength=0.555;

  int j, iy, im, id, ihmsf[4];
  double utc1, utc2, tai1, tai2, tt1, tt2, tcg1, tcg2, tcb1, tcb2, tdb1, tdb2, ut11, ut12, ut, dut=0, dat=0, dtr, dt, X, Y, s, xp, yp, dX, dY;

  time_t rawtime;
  struct tm *ptm;
 
  time(&rawtime);
  ptm = localtime(&rawtime);
  printf("local:      %02i.%02i.%i %02i:%02i:%02i\n", ptm->tm_mday, ptm->tm_mon+1, ptm->tm_year+1900, ptm->tm_hour, ptm->tm_min, ptm->tm_sec);
  ptm = gmtime(&rawtime);

  for(int i = 1; i<argc; i++){
    std::string str(argv[i]);
    size_t plz;

    plz = str.find_first_of(".");
    if(plz!=std::string::npos){
      ptm->tm_mday = atoi(str.substr(plz-2,2).c_str());
      ptm->tm_mon = atoi(str.substr(plz+1,2).c_str())-1;
      ptm->tm_year = atoi(str.substr(plz+4,4).c_str())-1900;
      ptm->tm_hour = 0;
      ptm->tm_min = 0;
      ptm->tm_sec = 0;
    }

    plz = str.find_first_of("/");
    if(plz!=std::string::npos){
      ptm->tm_year = atoi(str.substr(plz-4,4).c_str())-1900;
      ptm->tm_mon = atoi(str.substr(plz+1,2).c_str())-1;
      ptm->tm_mday = atoi(str.substr(plz+4,4).c_str());
      ptm->tm_hour = 0;
      ptm->tm_min = 0;
      ptm->tm_sec = 0;
    }

    plz = str.find_first_of(":");
    if(plz!=std::string::npos){
      ptm->tm_hour = atoi(str.substr(plz-2,2).c_str());
      ptm->tm_min = atoi(str.substr(plz+1,2).c_str());
      ptm->tm_sec = 0;
    }
    plz = str.find_first_of(":",plz+1);
    if(plz!=std::string::npos)
      ptm->tm_sec = atoi(str.substr(plz+1,2).c_str());

  }
  
  iy = ptm->tm_year+1900;
  im = ptm->tm_mon+1;
  id = ptm->tm_mday;
  ihmsf[0] = ptm->tm_hour;
  ihmsf[1] = ptm->tm_min;
  ihmsf[2] = ptm->tm_sec;
  ihmsf[3] = 0;

  j = iauDtf2d("UT1", iy, im, id, ihmsf[0], ihmsf[1], ihmsf[2], &utc1, &utc2);
  double mjd_utc = (utc1-2400000.5)+utc2;
  if(!j){
    dut = ReadDUT(mjd_utc,xp,yp,dX,dY);
    dat = ReadDAT(utc1+utc2, mjd_utc);
  }
  if(dut!=0 && dat!=0){
    printf("UTC:        %02i.%02i.%i %02i:%02i:%02i\n", id, im, iy, ihmsf[0], ihmsf[1], ihmsf[2]);

    ut11=utc1;
    ut12=utc2+dut/3600/24;
    j = iauD2dtf("UT1", 6, ut11, ut12, &iy, &im, &id, ihmsf);
    if(j) return 1;
    printf("UT1:        %02i.%02i.%04i %02i:%02i:%02i.%06i  (UT1-UTC: %.6fs)\n", id, im, iy, ihmsf[0], ihmsf[1], ihmsf[2], ihmsf[3], dut);

    tai1=utc1;
    tai2=utc2+dat/3600/24;
    j = iauD2dtf("TAI", 6, tai1, tai2, &iy, &im, &id, ihmsf);
    if(j) return 1;
    printf("TAI:        %02i.%02i.%04i %02i:%02i:%02i.%06i  (TAI-UTC: %.6fs)\n", id, im, iy, ihmsf[0], ihmsf[1], ihmsf[2], ihmsf[3], ((tai1-utc1)+(tai2-utc2))*24*3600);
    
    j = iauTaitt(tai1, tai2, &tt1, &tt2);
    if(j) return 1;
    j = iauD2dtf("TT", 6, tt1, tt2, &iy, &im, &id, ihmsf);
    if(j) return 1;
    dt = ((tt1-ut11)+(tt2-ut12))*24*3600;
  }else{
    printf("UT1:        %02i.%02i.%i %02i:%02i:%02i\n", id, im, iy, ihmsf[0], ihmsf[1], ihmsf[2]);

    ut11=utc1; ut12=utc2;
    tai1=utc1; tai2=utc2;
    xp=yp=dX=dY=0;
    dt = ReadDT(iy+(im-0.5)/12);

    j = iauUt1tt(ut11, ut12, dt, &tt1, &tt2);
    if(j) return 1;
    j = iauD2dtf("TT", 6, tt1, tt2, &iy, &im, &id, ihmsf);
    if(j) return 1;
  }
  printf("TT(=TDT):   %02i.%02i.%04i %02i:%02i:%02i.%06i  (TT-UT1:  %.6fs)\n", id, im, iy, ihmsf[0], ihmsf[1], ihmsf[2], ihmsf[3], dt);
  double ttcorr=ReadTTcorr(mjd_utc);
  tt2+=ttcorr/1000/1000/24/3600;
  j = iauD2dtf("TT", 6, tt1, tt2, &iy, &im, &id, ihmsf);
  if(j) return 1;
  dt = ((tt1-ut11)+(tt2-ut12))*24*3600;
  printf("TT(BIPMxx): %02i.%02i.%04i %02i:%02i:%02i.%06i  (TT-UT1:  %.6fs)\n", id, im, iy, ihmsf[0], ihmsf[1], ihmsf[2], ihmsf[3], dt);    

  j = iauTttcg(tt1, tt2, &tcg1, &tcg2);
  if(j) return 1;
  j = iauD2dtf("TCG", 6, tcg1, tcg2, &iy, &im, &id, ihmsf);
  if(j) return 1;
  printf("TCG:        %02i.%02i.%04i %02i:%02i:%02i.%06i\n", id, im, iy, ihmsf[0], ihmsf[1], ihmsf[2], ihmsf[3]);

  ut = fmod(fmod(ut11, 1.0) + fmod(ut12, 1.0), 1.0);
  dtr = iauDtdb(tt1, tt2, ut, 0, 0, 0);
  j = iauTttdb(tt1, tt2, dtr, &tdb1, &tdb2);
  if(j) return 1;
  dtr = iauDtdb(tdb1, tdb2, ut, 0, 0, 0);  //iterative method
  j = iauTttdb(tt1, tt2, dtr, &tdb1, &tdb2);
  if(j) return 1;
  j = iauD2dtf("TDB", 6, tdb1, tdb2, &iy, &im, &id, ihmsf);
  if(j) return 1;
  printf("TDB:        %02i.%02i.%04i %02i:%02i:%02i.%06i  (TDB-TT:  %6.6fms)\n", id, im, iy, ihmsf[0], ihmsf[1], ihmsf[2], ihmsf[3], dtr*1000);
  
  j = iauTdbtcb(tdb1, tdb2, &tcb1, &tcb2);
  if(j) return 1;
  j = iauD2dtf("TCB", 6, tcb1, tcb2, &iy, &im, &id, ihmsf);
  if(j) return 1;
  printf("TCB:        %02i.%02i.%04i %02i:%02i:%02i.%06i\n", id, im, iy, ihmsf[0], ihmsf[1], ihmsf[2], ihmsf[3]);

  printf("JD (TT):    %14.6f\n",tt1+tt2);
  printf("MJD (UTC):  %14.6f\n",mjd_utc);

  double era = iauEra00(ut11, ut12);
  ihmsf[0]=era/D2PI*24;
  ihmsf[1]=fmod(era/D2PI*24,1.0)*60;
  ihmsf[2]=fmod(era/D2PI*24*60,1.0)*60;
  ihmsf[3]=fmod(era/D2PI*24*3600,1.0)*1000000;
  printf("ERA:        %02ih %02im %02i.%06is\n", ihmsf[0], ihmsf[1], ihmsf[2], ihmsf[3]);
  
  iauXys06a(tt1, tt2, &X, &Y, &s); //X,Y and s from 2006 precession and 2000A nutation model
  double sp=iauSp00(tt1,tt2);
  printf("CIP @ GCRS: X=%f\" Y=%f\" dX=%f\" dY=%f\"\n",X*3600*180/DPI,Y*3600*180/DPI,dX*3600*180/DPI,dY*3600*180/DPI);
  printf("CIO @ GCRS: s=%fmas\n",s*1000*3600*180/DPI);
  printf("CIP @ ITRS: x=%f\" y=%f\"\n",xp*3600*180/DPI,yp*3600*180/DPI);
  printf("TIO @ ITRS: s'=%fmas\n",sp*1000*3600*180/DPI);
  X+=dX; //apply IERS correction of the 2000A nutation model
  Y+=dY; //apply IERS correction of the 2000A nutation model
    
  double pv[6], ebpv[2][3], ehp[3];
  // Barycentric coordinates and velocities of the earth
  calceph_compute_unit(peph, tdb1, tdb2, 3, 12, CALCEPH_UNIT_AU+CALCEPH_UNIT_DAY, pv);
  for(int i1=0; i1<2; i1++)
    for(int i2=0; i2<3; i2++)
      ebpv[i1][i2]=pv[i1*3+i2];

  // Heliocentric coordinates of the earth
  calceph_compute_unit(peph, tdb1, tdb2, 3, 11, CALCEPH_UNIT_AU+CALCEPH_UNIT_DAY, pv);
  for(int i2=0; i2<3; i2++)
    ehp[i2]=pv[i2];

  iauASTROM astrom;
  lat*=DPI/180;
  lon*=DPI/180;
  iauRefco(press,temp,relhum,wavelength,&astrom.refa,&astrom.refb);
  iauApco(tdb1,tdb2,ebpv,ehp,X,Y,s,era,lon,lat,elev,xp,yp,sp,astrom.refa,astrom.refb,&astrom);
  
  printf("\nGeocenter BCRS: x=%+010.7f y=%+010.7f z=%+010.7f\n",ebpv[0][0],ebpv[0][1],ebpv[0][2]);
  printf("Observer  BCRS: x=%+010.7f y=%+010.7f z=%+010.7f\n",astrom.eb[0],astrom.eb[1],astrom.eb[2]);
  
  std::string planetstr[]={"Sun","Moon","Venus","Mars","Jupiter","Saturn"};
  int planetidx[]={11,10,2,4,5,6};
  double au; calceph_getconstant(peph, "AU", &au);
  for(int i1=0; i1<sizeof(planetidx)/sizeof(planetidx[0]); i1++){
    double lighttime=0;
    for(int i2=0; i2<3; i2++){
      // find when to send out the light ray so that it arrives now
      calceph_compute_unit(peph, tdb1-lighttime, tdb2, planetidx[i1], 12, CALCEPH_UNIT_AU+CALCEPH_UNIT_DAY, pv);
      double dist=sqrt(pow(pv[0]-astrom.eb[0],2)+pow(pv[1]-astrom.eb[1],2)+pow(pv[2]-astrom.eb[2],2));
      dist*=au/299792.458/3600/24;
      lighttime+=dist-lighttime;
    }
    std::string lighttimestr="days";
    double lighttimeval=lighttime;
    if(lighttimeval*24<24){
      lighttimeval*=24;
      lighttimestr="hours";
    }
    if(lighttimeval*60<60){
      lighttimeval*=60;
      lighttimestr="minutes";
    }
    if(lighttimeval*60<60){
      lighttimeval*=60;
      lighttimestr="seconds";
    }
    printf("\n%s: (%f light-%s)\n",planetstr[i1].c_str(), lighttimeval, lighttimestr.c_str());

    calceph_compute_unit(peph, tdb1-lighttime, tdb2, planetidx[i1], 12, CALCEPH_UNIT_AU+CALCEPH_UNIT_DAY, pv);
    double tmpv[2][3];
    for(int i1=0; i1<2; i1++)
      for(int i2=0; i2<3; i2++)
	tmpv[i1][i2]=pv[i1*3+i2];

    double ra, dec, pmr, pmd, px, rv, aob,zob,hob;
    iauPvstar(tmpv,&ra,&dec,&pmr,&pmd,&px,&rv);
    printf("BCRS: RA: %09.6f dec: %+010.6f, x=%+010.7f y=%+010.7f z=%+010.7f\n",ra*12/DPI,dec*180/DPI,pv[0],pv[1],pv[2]);
    iauAtciq(ra,dec,0,0,px,0,&astrom,&ra,&dec);
    printf("CIRS: RA: %09.6f dec: %+010.6f\n",ra*12/DPI,dec*180/DPI);
    iauAtioq(ra,dec,&astrom,&aob,&zob,&hob,&dec,&ra);
    printf("obs:  RA: %09.6f dec: %+010.6f, Az: %09.5f Alt: %+010.6f\n",ra*12/DPI,dec*180/DPI,aob*180/DPI,90-zob*180/DPI);
  }
  return 0;
}

double ReadTTcorr(double mjd_utc){
  if(mjd_utc<43144) return 0; //before 01.01.1977 TDT/TT did not exist
  int imjd = mjd_utc, tmjd, fmjd=0, smjd=0;
  double dut1 = 0, dut2 = 0;

  std::string line;
  std::ifstream myfile2("corrections/TTBIPM.act");
  if(myfile2.is_open())
    {
      while(myfile2.good())
	{
	  getline(myfile2,line);
	  if(line.length()>24 && line.length()<27){
	    tmjd = atoi(line.substr(0,5).c_str());
	    if(tmjd <= imjd){
	      dut1 = atof(line.substr(17,-1).c_str());
	      fmjd = tmjd;
	      continue;
	    }
	    if(tmjd > imjd){
	      dut2 = atof(line.substr(17,-1).c_str());
	      smjd = tmjd;
	      break;
	    }
	  }
	}
      myfile2.close();
    }
  if(smjd==0) return dut1;
  return (mjd_utc-fmjd)/(smjd-fmjd)*dut2+(smjd-mjd_utc)/(smjd-fmjd)*dut1;
}

double ReadDAT(double jd_utc, double mjd_utc){
  double ijd = jd_utc, tjd, fmjd=0;
  double dat = 0, drift = 0;

  std::string line;
  std::ifstream myfile2("corrections/tai-utc.dat");
  if(myfile2.is_open())
    {
      while(myfile2.good())
	{
	  getline(myfile2,line);
	  if(line.length()>8){
	    tjd = atof(line.substr(17,9).c_str());
	    if(tjd <= ijd){
	      dat = atof(line.substr(37,11).c_str());
	      fmjd= atof(line.substr(60,5).c_str());
	      drift=atof(line.substr(70,9).c_str());
	      continue;
	    }
	  }
	}
      myfile2.close();
    }
  return dat+(mjd_utc-fmjd)*drift;
}

double ReadDUT(double mjd_utc, double &xp, double &yp, double &dX, double &dY){
  int imjd = mjd_utc, tmjd, fmjd=0, smjd=0;
  double dut1 = 0, dut2 = 0, x1, x2, y1, y2, X1, X2, Y1, Y2;
  std::string line;

  std::ifstream myfile2("corrections/eopc04_IAU2000.62-now");
  if(myfile2.is_open())
    {
      while(myfile2.good())
	{
	  getline(myfile2,line);
	  if(line.length()>80){
	    tmjd = atoi(line.substr(14,5).c_str());
	    if(tmjd == imjd && dut1==0){
	      x1 = atof(line.substr(21,9).c_str());
	      y1 = atof(line.substr(32,9).c_str());
	      X1 = atof(line.substr(67,9).c_str());
	      Y1 = atof(line.substr(78,9).c_str());
	      dut1 = atof(line.substr(43,10).c_str());
	      fmjd = tmjd;
	      continue;
	    }
	    if(tmjd == imjd+1){
	      x2 = atof(line.substr(21,9).c_str());
	      y2 = atof(line.substr(32,9).c_str());
	      X2 = atof(line.substr(67,9).c_str());
	      Y2 = atof(line.substr(78,9).c_str());
	      dut2 = atof(line.substr(43,10).c_str());
	      smjd = tmjd;
	      break;
	    }
	  }
	}
      myfile2.close();
    }

  std::ifstream myfile3("corrections/finals2000A.daily");
  if(dut2==0 && myfile3.is_open())
    {
      while(myfile3.good())
	{
	  getline(myfile3,line);
	  if(line.length()>80){
	    tmjd = atoi(line.substr(7,5).c_str());
	    if(tmjd == imjd && dut1==0){
	      x1 = atof(line.substr(18,9).c_str());
	      y1 = atof(line.substr(37,9).c_str());
	      X1 = atof(line.substr(97,9).c_str())/1000;
	      Y1 = atof(line.substr(116,9).c_str())/1000;
	      dut1 = atof(line.substr(58,10).c_str());
	      fmjd = tmjd;
	      continue;
	    }
	    if(tmjd == imjd+1){
	      x2 = atof(line.substr(18,9).c_str());
	      y2 = atof(line.substr(37,9).c_str());
	      X2 = atof(line.substr(97,9).c_str())/1000;
	      Y2 = atof(line.substr(116,9).c_str())/1000;
	      dut2 = atof(line.substr(58,10).c_str());
	      smjd = tmjd;
	      break;
	    }
	  }
	}
      myfile3.close();
    }

  std::ifstream myfile("corrections/finals2000A.data");
  if(dut2==0 && myfile.is_open())
    {
      while(myfile.good())
	{
	  getline(myfile,line);
	  if(line.length()>80){
	    tmjd = atoi(line.substr(7,5).c_str());
	    if(tmjd == imjd && dut1==0){
	      x1 = atof(line.substr(18,9).c_str());
	      y1 = atof(line.substr(37,9).c_str());
	      X1 = atof(line.substr(97,9).c_str())/1000;
	      Y1 = atof(line.substr(116,9).c_str())/1000;
	      dut1 = atof(line.substr(58,10).c_str());
	      fmjd = tmjd;
	      continue;
	    }
	    if(tmjd == imjd+1){
	      x2 = atof(line.substr(18,9).c_str());
	      y2 = atof(line.substr(37,9).c_str());
	      X2 = atof(line.substr(97,9).c_str())/1000;
	      Y2 = atof(line.substr(116,9).c_str())/1000;
	      dut2 = atof(line.substr(58,10).c_str());
	      smjd = tmjd;
	      break;
	    }
	  }
	}
      myfile.close();
    }

  if(smjd==0 || x2==0){ xp=x1; dX=X1; }
  else{
    xp=(mjd_utc-fmjd)/(smjd-fmjd)*x2+(smjd-mjd_utc)/(smjd-fmjd)*x1;
    dX=(mjd_utc-fmjd)/(smjd-fmjd)*X2+(smjd-mjd_utc)/(smjd-fmjd)*X1;
  }
  
  if(smjd==0 || y2==0){ yp=y1; dY=Y1; }
  else{
    yp=(mjd_utc-fmjd)/(smjd-fmjd)*y2+(smjd-mjd_utc)/(smjd-fmjd)*y1;
    dY=(mjd_utc-fmjd)/(smjd-fmjd)*Y2+(smjd-mjd_utc)/(smjd-fmjd)*Y1;
  }

  xp/=3600*180/DPI; //" to rad, to be compatible to sofa
  yp/=3600*180/DPI; //" to rad, to be compatible to sofa
  dX/=3600*180/DPI; //" to rad, to be compatible to sofa
  dY/=3600*180/DPI; //" to rad, to be compatible to sofa

  if(smjd==0 || dut2==0) return dut1;
  return (mjd_utc-fmjd)/(smjd-fmjd)*dut2+(smjd-mjd_utc)/(smjd-fmjd)*dut1;
}


double ReadDT(double year){
  double dt1=0, dt2=0, tyear, fyear=0, syear=0;
  std::string line;
  std::ifstream myfile2("corrections/deltat.preds");
  if(myfile2.is_open())
    {
      while(myfile2.good())
	{
	  getline(myfile2,line);
	  if(line.length()>20){
	    tyear = atof(line.substr(0,8).c_str());
	    if(year>tyear && year<=tyear+0.25 && dt1==0){
	      dt1 = atof(line.substr(12,7).c_str());
	      fyear = tyear;
	      continue;
	    }
	    if(year<=tyear+0.25 && dt1!=0){
	      dt2 = atof(line.substr(12,7).c_str());
	      syear = tyear;
	      break;
	    }
	  }
	}
      myfile2.close();
    }

  if(fyear==0 || dt1==0 || syear==0 || dt2==0){
    // from http://eclipse.gsfc.nasa.gov/SEhelp/deltatpoly2004.html
    // based on Morrison, L. and Stephenson, F. R., "Historical Values of the Earth's Clock Error DeltaT and the Calculation of Eclipses", J. Hist. Astron., Vol. 35 Part 3, August 2004, No. 120, pp 327-336 (2004).
    if(year<-500){
      double u=(year-1820)/100;
      return -20 + 32*u*u;
      
    }else if(year<500){
      double u=year/100;
      return 10583.6 + u*(-1014.41 + u*(33.78311 + u*(-5.952053 + u*(-0.1798452 + u*(0.022174192 + u*0.0090316521)))));
      
    }else if(year<1600){
      double u=(year-1000)/100;
      return 1574.2 + u*(-556.01 + u*(71.23472 + u*(0.319781 + u*(-0.8503463 + u*(-0.005050998 + u*0.0083572073)))));
      
    }else if(year<1700){
      double t=year-1600;
      return 120 + t*(-0.9808 + t*(-0.01532 + t/7129));
      
    }else if(year<1800){
      double t=year-1700;
      return 8.83 + t*(0.1603 + t*(-0.0059285 + t*(0.00013336 - t/1174000)));
      
    }else if(year<1860){
      double t=year-1800;
      return 13.72 + t*(-0.332447 + t*(0.0068612 + t*(0.0041116 + t*(-0.00037436 + t*(0.0000121272 + t*(-0.0000001699 + t*0.000000000875))))));
      
    }else if(year<1900){
      double t=year-1860;
      return 7.62 + t*(0.5737 + t*(-0.251754 + t*(0.01680668 + t*(-0.0004473624 + t/233174))));
      
    }else if(year<1920){
      double t=year-1900;
      return -2.79 + t*(1.494119 + t*(-0.0598939 + t*(0.0061966 - 0.000197 * t)));
      
    }else if(year<1941){
      double t=year-1920;
      return 21.20 + t*(0.84493 + t*(-0.076100 + 0.0020936 * t));
      
    }else if(year<1961){
      double t=year-1950;
      return 29.07 + t*(0.407 - t/233 + t*t/2547);
      
    }else if(year<1986){
      double t=year-1975;
      return 45.45 + t*(1.067 - t/260 - t*t/718);
      
    }else if(year<2005){
      double t=year-2000;
      return 63.86 + t*(0.3345 + t*(-0.060374 + t*(0.0017275 + t*(0.000651814 + 0.00002373599 * t))));
      
    }else if(year<2050){
      double t=year-2000;
      return 62.92 + t*(0.32217 + 0.005589 * t);
      
    }else if(year<2150){
      double u=(year-1820)/100;
      return -20 + 32*u*u - 0.5628 * (2150 - year);
      
    }else{
      double u=(year-1820)/100;
      return -20 + 32*u*u;
    }
  }else
    return (year-fyear)/(syear-fyear)*dt2+(syear-year)/(syear-fyear)*dt1;
}


void DownloadFiles(){
  printf("Searching for IERS files... \n");
  time_t rawtime;
  struct stat attrib;
  float fileage;
  size_t filesize;
  std::string filename;
  time(&rawtime);

  // //based on McCarthy and Babcock (1986)
  // fileage = std::numeric_limits<float>::infinity();
  // filesize= 0;
  // if(!stat("corrections/historic_deltat.data", &attrib)){
  //   fileage = difftime(rawtime,attrib.st_mtime)/24/3600;
  //   filesize= attrib.st_size;
  //   printf("historic_deltat.data found (%f days old)\n", fileage);
  // }
  // else
  //   printf("historic_deltat.data not found. Downloading...\n");
  // if(filesize==0 || fileage>180){
  //   if(!downloadFile("http://maia.usno.navy.mil/ser7/historic_deltat.data","corrections/historic_deltat.data"))
  //     downloadFile("http://toshi.nofs.navy.mil/ser7/historic_deltat.data","corrections/historic_deltat.data");
  // }

  fileage = std::numeric_limits<float>::infinity();
  filesize= 0;
  if(!stat("corrections/deltat.preds", &attrib)){
    fileage = difftime(rawtime,attrib.st_mtime)/24/3600;
    filesize= attrib.st_size;
    printf("deltat.preds found (%f days old)\n", fileage);
  }
  else
    printf("deltat.preds not found. Downloading...\n");
  if(filesize==0 || fileage>45){
    if(!downloadFile("http://maia.usno.navy.mil/ser7/deltat.preds","corrections/deltat.preds"))
      downloadFile("http://toshi.nofs.navy.mil/ser7/deltat.preds","corrections/deltat.preds");
  }

  fileage = std::numeric_limits<float>::infinity();
  filesize= 0;
  if(!stat("corrections/tai-utc.dat", &attrib)){
    fileage = difftime(rawtime,attrib.st_mtime)/24/3600;
    filesize= attrib.st_size;
    printf("tai-utc.dat found (%f days old)\n", fileage);
  }
  else
    printf("tai-utc.dat not found. Downloading...\n");
  if(filesize==0 || fileage>90){
    if(!downloadFile("http://maia.usno.navy.mil/ser7/tai-utc.dat","corrections/tai-utc.dat"))
      downloadFile("http://toshi.nofs.navy.mil/ser7/tai-utc.dat","corrections/tai-utc.dat");
  }

  fileage = std::numeric_limits<float>::infinity();
  filesize= 0;
  if(!stat("corrections/finals2000A.data", &attrib)){
    fileage = difftime(rawtime,attrib.st_mtime)/24/3600;
    filesize= attrib.st_size;
    printf("finals2000A.data found (%f days old)\n", fileage);
  }
  else
    printf("finals2000A.data not found. Downloading...\n");
  if(filesize==0 || fileage>7){
    if(!downloadFile("http://maia.usno.navy.mil/ser7/finals2000A.data","corrections/finals2000A.data",true))
      if(!downloadFile("http://toshi.nofs.navy.mil/ser7/finals2000A.data","corrections/finals2000A.data",true))
        downloadFile("ftp://ftp.iers.org/products/eop/rapid/standard/finals2000A.data","corrections/finals2000A.data",true);
  }

  fileage = std::numeric_limits<float>::infinity();
  filesize= 0;
  if(!stat("corrections/finals2000A.daily", &attrib)){
    fileage = difftime(rawtime,attrib.st_mtime)/24/3600;
    filesize= attrib.st_size;
    printf("finals2000A.daily found (%f days old)\n", fileage);
  }
  else
    printf("finals2000A.daily not found. Downloading...\n");
  if(filesize==0 || fileage>2){
    if(!downloadFile("http://maia.usno.navy.mil/ser7/finals2000A.daily","corrections/finals2000A.daily",true))
      if(!downloadFile("http://toshi.nofs.navy.mil/ser7/finals2000A.daily","corrections/finals2000A.daily",true))
        downloadFile("ftp://ftp.iers.org/products/eop/rapid/daily/finals2000A.daily","corrections/finals2000A.daily",true);
  }

  fileage = std::numeric_limits<float>::infinity();
  filesize= 0;
  if(!stat("corrections/eopc04_IAU2000.62-now", &attrib)){
    fileage = difftime(rawtime,attrib.st_mtime)/24/3600;
    filesize= attrib.st_size;
    printf("eopc04_IAU2000.62-now found (%f days old)\n", fileage);
  }
  else
    printf("eopc04_IAU2000.62-now not found. Downloading...\n");
  if(filesize==0 || fileage>7){
    if(!downloadFile("ftp://hpiers.obspm.fr/iers/eop/eopc04/eopc04_IAU2000.62-now","corrections/eopc04_IAU2000.62-now",true))
      downloadFile("ftp://ftp.iers.org/products/eop/long-term/c04_14/iau2000/eopc04_14_IAU2000.62-now","corrections/eopc04_IAU2000.62-now",true);
  }
  
  fileage = std::numeric_limits<float>::infinity();
  filesize= 0;
  if(!stat("corrections/TTBIPM.act", &attrib)){
    fileage = difftime(rawtime,attrib.st_mtime)/24/3600;
    filesize= attrib.st_size;
    printf("TTBIPM.act found (%f days old)\n", fileage);
  }
  else
    printf("TTBIPM.act not found. Downloading...\n");
  if(filesize==0 || fileage>30){
    std::string url = "ftp://ftp2.bipm.org/pub/tai/ttbipm/";
    listFtp(url.data());
    url+=readFtpList("TTBIPM");
    downloadFile(url.data(),"corrections/TTBIPM.act");
  }

  // fileage = std::numeric_limits<float>::infinity();
  // filesize= 0;
  // if(!stat("corrections/ealtai_act.ar", &attrib)){
  //   fileage = difftime(rawtime,attrib.st_mtime)/24/3600;
  //   filesize= attrib.st_size;
  //   printf("ealtai_act.ar found (%f days old)\n", fileage);
  // }
  // else
  //   printf("ealtai_act.ar not found. Downloading...\n");
  // if(filesize==0 || fileage>30){
  //   std::string url = "ftp://ftp2.bipm.org/pub/tai/scale/EALTAI/";
  //   listFtp(url.data());
  //   url+=readFtpList("ealtai");
  //   downloadFile(url.data(),"corrections/ealtai_act.ar");
  // }
}

void DownloadEphemerides(){
  //this is not necessarty for normal use
  printf("Searching for Ephemeris files... \n");
  time_t rawtime;
  struct stat attrib;
  float fileage;
  size_t filesize;
  std::string filename;
  time(&rawtime);

  // download ephemeris files. tested with: de430.bsp, de431.bsp, de432.bsp or de432s.bsp
  fileage = std::numeric_limits<float>::infinity();
  filesize= 0;
  if(!stat("data/inpop13c_TDB_m100_p100_tt.dat", &attrib)){
    fileage = difftime(rawtime,attrib.st_mtime)/24/3600;
    filesize= attrib.st_size;
    printf("inpop13c_TDB_m100_p100_tt.dat found (%f days old)\n", fileage);
  }
  else
    printf("inpop13c_TDB_m100_p100_tt.dat not found. Downloading...\n");
  if(filesize==0){
    downloadFile("ftp://ftp.imcce.fr/pub/ephem/planets/inpop13c/inpop13c_TDB_m100_p100_tt.dat","data/inpop13c_TDB_m100_p100_tt.dat");
  }
}

void ShowConstants(t_calcephbin *peph){
  double valueconstant;
  char nameconstant[CALCEPH_MAX_CONSTANTNAME];
  for(int i1=0; i1<calceph_getconstantcount(peph); i1++){
    calceph_getconstantindex(peph,i1,nameconstant,&valueconstant);
    printf("%i %s=%f\n",i1,nameconstant,valueconstant);
  }
}
